# -*- coding: utf-8 -*-

from odoo import models, api, fields
import hashlib


class ChallengeUser(models.Model):
    _name = "challenge.user"
    _description = "Odoo Recruitment Challenge"
    _rec_name = 'login'

    login = fields.Char(string='Login', required=True)
    key = fields.Char(string='Token', help='auth token', compute='_get_token')
    attempt_ids = fields.One2many('challenge.attempt', 'user_id', string="Attempts")
    attempt_success_ids = fields.One2many('challenge.attempt', 'user_id', domain=[('success', '=', True), ('active', '=', True)], string="Attempts Succeded")
    attempt_mean = fields.Integer(string='Attempts Mean', aggregator="avg", compute="_get_attempts_mean", store=True) # store for graph view
    attempts_count = fields.Integer(compute='_count_attempts', string="# Attempts", store=True)
    level_id = fields.Many2one('challenge.level', aggregator="avg", compute='_get_current_level_id', string='Level id')
    level = fields.Integer(string='Last level', aggregator="avg", compute='_get_max_level', store=True) # store for graph view
    country_code = fields.Char('Country code')
    active = fields.Boolean(default=True)


    @api.depends('login')
    def _get_token(self):
        for u in self:
            u.key = hashlib.sha1((u"DO_%s_%s_DD" % (u.id, u.login)).encode()).hexdigest()


    @api.depends('attempt_ids', 'attempt_ids.active')
    def _count_attempts(self):
        for u in self:
            u.attempts_count = len(u.attempt_ids)


    @api.depends('attempt_success_ids', 'attempt_success_ids.active', 'level_id')
    def _get_max_level(self):
        for u in self:
            u.level = max(u.attempt_success_ids.mapped('level_score') or [0])


    @api.depends('attempt_success_ids', 'attempt_success_ids.active')
    def _get_current_level_id(self):
        for u in self:
            all_attemps = u.attempt_success_ids.mapped(lambda a: (a.level_score, a.level_id))
            if all_attemps:
                u.level_id = max(all_attemps)[1]
            else:
                u.level_id = False


    @api.depends('attempt_ids', 'attempt_ids.active')
    def _get_attempts_mean(self):
        for u in self:
            u.attempt_mean = int(len(u.attempt_ids) / (len(u.level_id.previous_levels()) or 1))


class ChallengeAttempt(models.Model):
    _name = "challenge.attempt"
    _description = "Challenge Attempt"

    user_id = fields.Many2one('challenge.user', string='User')
    level_id = fields.Many2one('challenge.level', string='Level id')
    level_score = fields.Integer(related='level_id.sequence', string='Level')
    proposed = fields.Char(string='Solution from challenger')
    solution = fields.Char(string='Real Solution')
    success = fields.Boolean(string='Found ?')
    succeed_attempts = fields.Integer(string='Success after ... attempts', compute='_compute_succeed_attempts')
    time_elapsed = fields.Integer(string='Elapsed Min', compute='_compute_time', store=True)
    active = fields.Boolean(default=True)

    @api.depends('user_id.attempt_ids', 'level_id')
    def _compute_succeed_attempts(self):
        for rec in self:
            rec.succeed_attempts = len(rec.user_id.attempt_ids.filtered(lambda x: x.level_id == rec.level_id))

    @api.depends('level_id')
    def _compute_time(self):
        for rec in self:
            prev_attempt = rec.user_id.attempt_success_ids.filtered(lambda x: x.level_id == rec.level_id.prev_level())
            new = fields.Datetime.from_string(fields.Datetime.now())
            if not prev_attempt:
                old = fields.Datetime.from_string(rec.user_id.create_date)
            else:
                old = fields.Datetime.from_string(prev_attempt[0].create_date)
            rec.time_elapsed = int((new - old).seconds / 60)


class ChallengeLevel(models.Model):
    _name = "challenge.level"
    _description = "Challenge Level"
    _order = 'sequence asc'

    name = fields.Char('Name', required=True)
    tech_name = fields.Char('Technical Name', help='Used for controller', required=True)
    sequence = fields.Integer('Level')
    description = fields.Text("Help")
    hint = fields.Text("Hint")
    send_mail = fields.Boolean(default=False)

    _sql_constraints = [
        ('tech_name', 'unique (tech_name)', "Tech Name should be uniq"),
    ]

    def prev_level(self):
        n = self.search([('sequence', '<', self.sequence)], order="sequence desc", limit=1)
        return n and n[0]

    def next_level(self):
        n = self.search([('sequence', '>', self.sequence)], limit=1)
        return n and n[0]

    def previous_levels(self):
        return self.search([('sequence', '<=', self.sequence)]).mapped(lambda x: (x.name, x.id, x.sequence))
